@@ -1,6 +1,6 @@
 /*
- * Copyright (c) 2002-2010 BalaBit IT Ltd, Budapest, Hungary
- * Copyright (c) 1998-2010 Balázs Scheidler
+ * Copyright (c) 2002-2012 BalaBit IT Ltd, Budapest, Hungary
+ * Copyright (c) 1998-2012 Balázs Scheidler
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -26,10 +26,10 @@
 #include "syslog-names.h"
 #include "messages.h"
 #include "cfg.h"
-#include "logprocess.h"
 #include "gsocket.h"
 #include "misc.h"
 #include "tags.h"
+#include "cfg-tree.h"
 #include "filter-expr-grammar.h"
 
 #include <regex.h>
@@ -46,6 +46,12 @@ filter_expr_node_init(FilterExprNode *self)
   self->ref_cnt = 1;
 }
 
+/*
+ * In case the filter would modify the message the caller has to make sure
+ * that the message is writable.  You can always archieve that with
+ * filter_expr_eval_root() below, but you have to be on a processing path to
+ * do that.
+ */
 gboolean
 filter_expr_eval_with_context(FilterExprNode *self, LogMessage **msg, gint num_msg)
 {
@@ -53,8 +59,8 @@ filter_expr_eval_with_context(FilterExprNode *self, LogMessage **msg, gint num_m
 
   res = self->eval(self, msg, num_msg);
   msg_debug("Filter node evaluation result",
-            evt_tag_str("filter_result", res ? "match" : "not-match"),
-            evt_tag_str("filter_type", self->type),
+            evt_tag_str("result", res ? "match" : "not-match"),
+            evt_tag_str("type", self->type),
             NULL);
   return res;
 }
@@ -65,6 +71,21 @@ filter_expr_eval(FilterExprNode *self, LogMessage *msg)
   return filter_expr_eval_with_context(self, &msg, 1);
 }
 
+gboolean
+filter_expr_eval_root_with_context(FilterExprNode *self, LogMessage **msg, gint num_msg, const LogPathOptions *path_options)
+{
+  if (self->modify)
+    log_msg_make_writable(&msg[0], path_options);
+
+  return filter_expr_eval_with_context(self, msg, num_msg);
+}
+
+gboolean
+filter_expr_eval_root(FilterExprNode *self, LogMessage **msg, const LogPathOptions *path_options)
+{
+  return filter_expr_eval_root_with_context(self, msg, 1, path_options);
+}
+
 FilterExprNode *
 filter_expr_ref(FilterExprNode *self)
 {
@@ -98,6 +119,7 @@ fop_init(FilterExprNode *s, GlobalConfig *cfg)
     self->left->init(self->left, cfg);
   if (self->right && self->right->init)
     self->right->init(self->right, cfg);
+  self->super.modify = self->left->modify || self->right->modify;
 }
 
 static void
@@ -132,7 +154,6 @@ fop_or_new(FilterExprNode *e1, FilterExprNode *e2)
   
   fop_init_instance(self);
   self->super.eval = fop_or_eval;
-  self->super.modify = e1->modify || e2->modify;
   self->left = e1;
   self->right = e2;
   self->super.type = "OR";
@@ -154,7 +175,6 @@ fop_and_new(FilterExprNode *e1, FilterExprNode *e2)
   
   fop_init_instance(self);
   self->super.eval = fop_and_eval;
-  self->super.modify = e1->modify || e2->modify;
   self->left = e1;
   self->right = e2;
   self->super.type = "AND";
@@ -497,12 +517,19 @@ static void
 filter_call_init(FilterExprNode *s, GlobalConfig *cfg)
 {
   FilterCall *self = (FilterCall *) s;
-  LogProcessRule *rule;
+  LogExprNode *rule;
 
-  rule = g_hash_table_lookup(cfg->filters, self->rule);
+  rule = cfg_tree_get_object(&cfg->tree, ENC_FILTER, self->rule);
   if (rule)
     {
-      self->filter_expr = ((LogFilterPipe *) rule->head->data)->expr;
+      /* this is quite fragile and would break whenever the parsing code in
+       * cfg-grammar.y changes to parse a filter rule.  We assume that a
+       * filter rule has a single child, which contains a LogFilterPipe
+       * instance as its object. */
+
+
+      self->filter_expr = ((LogFilterPipe *) rule->children->object)->expr;
+      self->super.modify = self->filter_expr->modify;
     }
   else
     {
@@ -673,9 +700,12 @@ static gboolean
 log_filter_pipe_init(LogPipe *s)
 {
   LogFilterPipe *self = (LogFilterPipe *) s;
+  GlobalConfig *cfg = log_pipe_get_config(s);
 
   if (self->expr->init)
     self->expr->init(self->expr, log_pipe_get_config(s));
+  if (!self->name)
+    self->name = cfg_tree_get_rule_name(&cfg->tree, ENC_FILTER, s->expr_node);
   return TRUE;
 }
 
@@ -683,15 +713,19 @@ static void
 log_filter_pipe_queue(LogPipe *s, LogMessage *msg, const LogPathOptions *path_options, gpointer user_data)
 {
   LogFilterPipe *self = (LogFilterPipe *) s;
+  gchar buf[128];
   gboolean res;
   
   msg_debug("Filter rule evaluation begins",
-            evt_tag_str("filter_rule", self->name ? self->name : "unnamed"),
+            evt_tag_str("rule", self->name),
+            evt_tag_str("location", log_expr_node_format_location(s->expr_node, buf, sizeof(buf))),
             NULL);
-  res = filter_expr_eval(self->expr, msg);
+
+  res = filter_expr_eval_root(self->expr, &msg, path_options);
   msg_debug("Filter rule evaluation result",
-            evt_tag_str("filter_result", res ? "match" : "not-match"),
-            evt_tag_str("filter_rule", self->name ? self->name : "unnamed"),
+            evt_tag_str("result", res ? "match" : "not-match"),
+            evt_tag_str("rule", self->name),
+            evt_tag_str("location", log_expr_node_format_location(s->expr_node, buf, sizeof(buf))),
             NULL);
   if (res)
     {
@@ -706,11 +740,11 @@ log_filter_pipe_queue(LogPipe *s, LogMessage *msg, const LogPathOptions *path_op
 }
 
 static LogPipe *
-log_filter_pipe_clone(LogProcessPipe *s)
+log_filter_pipe_clone(LogPipe *s)
 {
   LogFilterPipe *self = (LogFilterPipe *) s;
 
-  return log_filter_pipe_new(filter_expr_ref(self->expr), self->name);
+  return log_filter_pipe_new(filter_expr_ref(self->expr));
 }
 
 static void
@@ -720,21 +754,19 @@ log_filter_pipe_free(LogPipe *s)
 
   g_free(self->name);
   filter_expr_unref(self->expr);
-  log_process_pipe_free_method(s);
+  log_pipe_free_method(s);
 }
 
 LogPipe *
-log_filter_pipe_new(FilterExprNode *expr, const gchar *name)
+log_filter_pipe_new(FilterExprNode *expr)
 {
   LogFilterPipe *self = g_new0(LogFilterPipe, 1);
 
-  log_process_pipe_init_instance(&self->super);
-  self->super.super.init = log_filter_pipe_init;
-  self->super.super.queue = log_filter_pipe_queue;
-  self->super.super.free_fn = log_filter_pipe_free;
-  self->super.super.flags |= expr->modify ? PIF_CLONE : 0;
+  log_pipe_init_instance(&self->super);
+  self->super.init = log_filter_pipe_init;
+  self->super.queue = log_filter_pipe_queue;
+  self->super.free_fn = log_filter_pipe_free;
   self->super.clone = log_filter_pipe_clone;
   self->expr = expr;
-  self->name = g_strdup(name);
-  return &self->super.super;
+  return &self->super;
 }
