@@ -1,10 +1,8 @@
+/* A Bison parser, made by GNU Bison 2.5.  */
 
-/* A Bison parser, made by GNU Bison 2.4.1.  */
-
-/* Skeleton interface for Bison's Yacc-like parsers in C
+/* Bison interface for Yacc-like parsers in C
    
-      Copyright (C) 1984, 1989, 1990, 2000, 2001, 2002, 2003, 2004, 2005, 2006
-   Free Software Foundation, Inc.
+      Copyright (C) 1984, 1989-1990, 2000-2011 Free Software Foundation, Inc.
    
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -49,9 +47,14 @@
      LL_CONTEXT_BLOCK_DEF = 8,
      LL_CONTEXT_BLOCK_REF = 9,
      LL_CONTEXT_BLOCK_CONTENT = 10,
-     LL_CONTEXT_PRAGMA = 11,
-     LL_CONTEXT_FORMAT = 12,
-     LL_CONTEXT_TEMPLATE_FUNC = 13,
+     LL_CONTEXT_BLOCK_ARG = 11,
+     LL_CONTEXT_PRAGMA = 12,
+     LL_CONTEXT_FORMAT = 13,
+     LL_CONTEXT_TEMPLATE_FUNC = 14,
+     LL_CONTEXT_INNER_DEST = 15,
+     LL_CONTEXT_INNER_SRC = 16,
+     LL_CONTEXT_CLIENT_PROTO = 17,
+     LL_CONTEXT_SERVER_PROTO = 18,
      KW_SOURCE = 10000,
      KW_FILTER = 10001,
      KW_PARSER = 10002,
@@ -60,6 +63,8 @@
      KW_OPTIONS = 10005,
      KW_INCLUDE = 10006,
      KW_BLOCK = 10007,
+     KW_JUNCTION = 10008,
+     KW_CHANNEL = 10009,
      KW_INTERNAL = 10010,
      KW_FILE = 10011,
      KW_SQL = 10030,
@@ -87,6 +92,7 @@
      KW_LOG_MSG_SIZE = 10077,
      KW_FILE_TEMPLATE = 10078,
      KW_PROTO_TEMPLATE = 10079,
+     KW_MARK_MODE = 10080,
      KW_CHAIN_HOSTNAMES = 10090,
      KW_NORMALIZE_HOSTNAMES = 10091,
      KW_KEEP_HOSTNAME = 10092,
@@ -111,6 +117,7 @@
      KW_PROGRAM_OVERRIDE = 10165,
      KW_HOST_OVERRIDE = 10166,
      KW_THROTTLE = 10170,
+     KW_THREADED = 10171,
      KW_FLAGS = 10190,
      KW_PAD_SIZE = 10200,
      KW_TIME_ZONE = 10201,
@@ -160,7 +167,20 @@
      LL_BLOCK = 10426,
      LL_PRAGMA = 10427,
      LL_EOL = 10428,
-     LL_ERROR = 10429
+     LL_ERROR = 10429,
+     KW_VALUE_PAIRS = 10500,
+     KW_SELECT = 10501,
+     KW_EXCLUDE = 10502,
+     KW_PAIR = 10503,
+     KW_KEY = 10504,
+     KW_SCOPE = 10505,
+     KW_SHIFT = 10506,
+     KW_REKEY = 10507,
+     KW_ADD_PREFIX = 10508,
+     KW_REPLACE = 10509,
+     KW_DEFAULT = 10511,
+     KW_RETRIES = 10512,
+     KW_DBD_OPTION = 10513
    };
 #endif
 /* Tokens.  */
@@ -174,9 +194,14 @@
 #define LL_CONTEXT_BLOCK_DEF 8
 #define LL_CONTEXT_BLOCK_REF 9
 #define LL_CONTEXT_BLOCK_CONTENT 10
-#define LL_CONTEXT_PRAGMA 11
-#define LL_CONTEXT_FORMAT 12
-#define LL_CONTEXT_TEMPLATE_FUNC 13
+#define LL_CONTEXT_BLOCK_ARG 11
+#define LL_CONTEXT_PRAGMA 12
+#define LL_CONTEXT_FORMAT 13
+#define LL_CONTEXT_TEMPLATE_FUNC 14
+#define LL_CONTEXT_INNER_DEST 15
+#define LL_CONTEXT_INNER_SRC 16
+#define LL_CONTEXT_CLIENT_PROTO 17
+#define LL_CONTEXT_SERVER_PROTO 18
 #define KW_SOURCE 10000
 #define KW_FILTER 10001
 #define KW_PARSER 10002
@@ -185,6 +210,8 @@
 #define KW_OPTIONS 10005
 #define KW_INCLUDE 10006
 #define KW_BLOCK 10007
+#define KW_JUNCTION 10008
+#define KW_CHANNEL 10009
 #define KW_INTERNAL 10010
 #define KW_FILE 10011
 #define KW_SQL 10030
@@ -212,6 +239,7 @@
 #define KW_LOG_MSG_SIZE 10077
 #define KW_FILE_TEMPLATE 10078
 #define KW_PROTO_TEMPLATE 10079
+#define KW_MARK_MODE 10080
 #define KW_CHAIN_HOSTNAMES 10090
 #define KW_NORMALIZE_HOSTNAMES 10091
 #define KW_KEEP_HOSTNAME 10092
@@ -236,6 +264,7 @@
 #define KW_PROGRAM_OVERRIDE 10165
 #define KW_HOST_OVERRIDE 10166
 #define KW_THROTTLE 10170
+#define KW_THREADED 10171
 #define KW_FLAGS 10190
 #define KW_PAD_SIZE 10200
 #define KW_TIME_ZONE 10201
@@ -286,6 +315,19 @@
 #define LL_PRAGMA 10427
 #define LL_EOL 10428
 #define LL_ERROR 10429
+#define KW_VALUE_PAIRS 10500
+#define KW_SELECT 10501
+#define KW_EXCLUDE 10502
+#define KW_PAIR 10503
+#define KW_KEY 10504
+#define KW_SCOPE 10505
+#define KW_SHIFT 10506
+#define KW_REKEY 10507
+#define KW_ADD_PREFIX 10508
+#define KW_REPLACE 10509
+#define KW_DEFAULT 10511
+#define KW_RETRIES 10512
+#define KW_DBD_OPTION 10513
 
 
 
