@@ -1,18 +1,18 @@
 /*
- * Copyright (c) 2002-2010 BalaBit IT Ltd, Budapest, Hungary
- * Copyright (c) 1998-2010 Balázs Scheidler
+ * Copyright (c) 2002-2012 BalaBit IT Ltd, Budapest, Hungary
+ * Copyright (c) 1998-2012 Balázs Scheidler
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
  * by the Free Software Foundation, or (at your option) any later version.
  *
- * This library is distributed in the hope that it will be useful,
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  *
  * As an additional exemption you are allowed to compile & link against the
@@ -27,6 +27,7 @@
 #include "timeutils.h"
 #include "misc.h"
 #include "cfg.h"
+#include "str-format.h"
 
 #include <regex.h>
 #include <ctype.h>
@@ -44,13 +45,8 @@
 
 static const char aix_fwd_string[] = "Message forwarded from ";
 static const char repeat_msg_string[] = "last message repeated";
-
-static inline void
-copy_string_fixed_buf(guchar *dest, const guchar *src, gsize dest_len)
-{
-  strncpy((gchar *) dest, (gchar *) src, dest_len);
-  dest[dest_len-1] = 0;
-}
+static NVHandle is_synced;
+static NVHandle cisco_seqid;
 
 static gboolean
 log_msg_parse_pri(LogMessage *self, const guchar **data, gint *length, guint flags, guint16 default_pri)
@@ -186,48 +182,84 @@ log_msg_parse_column(LogMessage *self, NVHandle handle, const guchar **data, gin
   *length = left;
 }
 
-
 static gboolean
-log_msg_parse_date(LogMessage *self, const guchar **data, gint *length, guchar *date, gsize date_len, guint parse_flags, glong assume_timezone)
+log_msg_parse_seq(LogMessage *self, const guchar **data, gint *length)
 {
   const guchar *src = *data;
   gint left = *length;
-  time_t now = time(NULL);
+
+
+  while (left && *src != ':')
+    {
+      if (!isdigit(*src))
+          return FALSE;
+      src++;
+      left--;
+    }
+  src++;
+  left--;
+
+  /* if the next char is not space, then we may try to read a date */
+
+  if (*src != ' ')
+    return FALSE;
+
+  log_msg_set_value(self, cisco_seqid, (gchar *) *data, *length - left - 1);
+
+  *data = src;
+  *length = left;
+  return TRUE;
+}
+
+/* FIXME: this function should really be exploded to a lot of smaller functions... (Bazsi) */
+static gboolean
+log_msg_parse_date(LogMessage *self, const guchar **data, gint *length, guint parse_flags, glong assume_timezone)
+{
+  const guchar *src = *data;
+  gint left = *length;
+  GTimeVal now;
   struct tm tm;
-  guchar *p;
   gint unnormalized_hour;
 
-  date[0] = 0;
+  cached_g_current_time(&now);
 
+  if ((parse_flags & LP_SYSLOG_PROTOCOL) == 0)
+    {
+      /* Cisco timestamp extensions, the first '*' indicates that the clock is
+       * unsynced, '.' if it is known to be synced */
+      if (G_UNLIKELY(src[0] == '*'))
+        {
+          log_msg_set_value(self, is_synced, "0", 1);
+          src++;
+          left--;
+        }
+      else if (G_UNLIKELY(src[0] == '.'))
+        {
+          log_msg_set_value(self, is_synced, "1", 1);
+          src++;
+          left--;
+        }
+    }
   /* If the next chars look like a date, then read them as a date. */
   if (left >= 19 && src[4] == '-' && src[7] == '-' && src[10] == 'T' && src[13] == ':' && src[16] == ':')
     {
       /* RFC3339 timestamp, expected format: YYYY-MM-DDTHH:MM:SS[.frac]<+/->ZZ:ZZ */
       gint hours, mins;
 
-      self->timestamps[LM_TS_STAMP].time.tv_usec = 0;
-
-      copy_string_fixed_buf(date, src, MIN(date_len, 20));
+      self->timestamps[LM_TS_STAMP].tv_usec = 0;
 
       /* NOTE: we initialize various unportable fields in tm using a
        * localtime call, as the value of tm_gmtoff does matter but it does
        * not exist on all platforms and 0 initializing it causes trouble on
        * time-zone barriers */
 
-      cached_localtime(&now, &tm);
-      p = (guchar *) strptime((gchar *) date, STRPTIME_ISOFORMAT, &tm);
-
-      if (!p || (p && *p))
+      cached_localtime(&now.tv_sec, &tm);
+      if (!scan_iso_timestamp((const gchar **) &src, &left, &tm))
         {
-          /* not the complete stamp could be parsed */
           goto error;
         }
 
-      src += p - date;
-      left -= p - date;
-
-
-      self->timestamps[LM_TS_STAMP].time.tv_usec = 0;
+      self->timestamps[LM_TS_STAMP].tv_usec = 0;
       if (left > 0 && *src == '.')
         {
           gulong frac = 0;
@@ -248,7 +280,7 @@ log_msg_parse_date(LogMessage *self, const guchar **data, gint *length, guchar *
               src++;
               left--;
             }
-          self->timestamps[LM_TS_STAMP].time.tv_usec = frac * (1000000 / div);
+          self->timestamps[LM_TS_STAMP].tv_usec = frac * (1000000 / div);
         }
 
       if (left > 0 && *src == 'Z')
@@ -274,86 +306,47 @@ log_msg_parse_date(LogMessage *self, const guchar **data, gint *length, guchar *
 
       tm.tm_isdst = -1;
       unnormalized_hour = tm.tm_hour;
-      self->timestamps[LM_TS_STAMP].time.tv_sec = cached_mktime(&tm);
+      self->timestamps[LM_TS_STAMP].tv_sec = cached_mktime(&tm);
     }
   else if ((parse_flags & LP_SYSLOG_PROTOCOL) == 0)
     {
-      if (left >= 21 && src[3] == ' ' && src[6] == ' ' && src[11] == ' ' && src[14] == ':' && src[17] == ':' && src[20] == ':' &&
+      if (left >= 21 && src[3] == ' ' && src[6] == ' ' && src[11] == ' ' && src[14] == ':' && src[17] == ':' && (src[20] == ':' || src[20] == ' ') &&
           (isdigit(src[7]) && isdigit(src[8]) && isdigit(src[9]) && isdigit(src[10])))
         {
           /* PIX timestamp, expected format: MMM DD YYYY HH:MM:SS: */
-
-          /* Just read the buffer data into a textual
-             datestamp. */
-
-          copy_string_fixed_buf(date, src, MIN(date_len, 22));
-          src += 21;
-          left -= 21;
-
-          /* And also make struct time timestamp for the msg */
-
-          cached_localtime(&now, &tm);
-          p = (guchar *) strptime((gchar *) date, "%b %e %Y %H:%M:%S:", &tm);
-          if (!p || (p && *p))
-            goto error;
-
-          tm.tm_isdst = -1;
-
-          /* NOTE: no timezone information in the message, assume it is local time */
-          unnormalized_hour = tm.tm_hour;
-          self->timestamps[LM_TS_STAMP].time.tv_sec = cached_mktime(&tm);
-          self->timestamps[LM_TS_STAMP].time.tv_usec = 0;
-        }
-      else if (left >= 21 && src[3] == ' ' && src[6] == ' ' && src[11] == ' ' && src[14] == ':' && src[17] == ':' && src[20] == ' ' &&
-          (isdigit(src[7]) && isdigit(src[8]) && isdigit(src[9]) && isdigit(src[10])))
-        {
           /* ASA timestamp, expected format: MMM DD YYYY HH:MM:SS */
 
-          /* Just read the buffer data into a textual
-             datestamp. */
-
-          copy_string_fixed_buf(date, src, MIN(date_len, 21));
-          src += 20;
-          left -= 20;
-
-          /* And also make struct time timestamp for the msg */
-
-          cached_localtime(&now, &tm);
-          p = (guchar *) strptime((gchar *) date, "%b %e %Y %H:%M:%S", &tm);
-          if (!p || (p && *p))
+          cached_localtime(&now.tv_sec, &tm);
+          if (!scan_pix_timestamp((const gchar **) &src, &left, &tm))
             goto error;
 
+          if (*src == ':')
+            {
+              src++;
+              left--;
+            }
+
           tm.tm_isdst = -1;
 
           /* NOTE: no timezone information in the message, assume it is local time */
           unnormalized_hour = tm.tm_hour;
-          self->timestamps[LM_TS_STAMP].time.tv_sec = cached_mktime(&tm);
-          self->timestamps[LM_TS_STAMP].time.tv_usec = 0;
+          self->timestamps[LM_TS_STAMP].tv_sec = cached_mktime(&tm);
+          self->timestamps[LM_TS_STAMP].tv_usec = 0;
         }
       else if (left >= 21 && src[3] == ' ' && src[6] == ' ' && src[9] == ':' && src[12] == ':' && src[15] == ' ' &&
                isdigit(src[16]) && isdigit(src[17]) && isdigit(src[18]) && isdigit(src[19]) && isspace(src[20]))
         {
           /* LinkSys timestamp, expected format: MMM DD HH:MM:SS YYYY */
 
-          /* Just read the buffer data into a textual
-             datestamp. */
-
-          copy_string_fixed_buf(date, src, 21);
-          src += 20;
-          left -= 20;
-
-          /* And also make struct time timestamp for the msg */
-
-          cached_localtime(&now, &tm);
-          p = (guchar *) strptime((gchar *) date, "%b %e %H:%M:%S %Y", &tm);
-          if (!p || (p && *p))
+          cached_localtime(&now.tv_sec, &tm);
+          if (!scan_linksys_timestamp((const gchar **) &src, &left, &tm))
             goto error;
           tm.tm_isdst = -1;
 
           /* NOTE: no timezone information in the message, assume it is local time */
           unnormalized_hour = tm.tm_hour;
-          self->timestamps[LM_TS_STAMP].time.tv_sec = cached_mktime(&tm);
-          self->timestamps[LM_TS_STAMP].time.tv_usec = 0;
+          self->timestamps[LM_TS_STAMP].tv_sec = cached_mktime(&tm);
+          self->timestamps[LM_TS_STAMP].tv_usec = 0;
 
         }
       else if (left >= 15 && src[3] == ' ' && src[6] == ' ' && src[9] == ':' && src[12] == ':')
@@ -362,12 +355,10 @@ log_msg_parse_date(LogMessage *self, const guchar **data, gint *length, guchar *
           struct tm nowtm;
           glong usec = 0;
 
-          /* Just read the buffer data into a textual
-             datestamp. */
-
-          copy_string_fixed_buf(date, src, MIN(date_len, 16));
-          src += 15;
-          left -= 15;
+          cached_localtime(&now.tv_sec, &nowtm);
+          tm = nowtm;
+          if (!scan_bsd_timestamp((const gchar **) &src, &left, &tm))
+            goto error;
 
           if (left > 0 && src[0] == '.')
             {
@@ -391,28 +382,23 @@ log_msg_parse_date(LogMessage *self, const guchar **data, gint *length, guchar *
               src += i;
             }
 
-          /* And also make struct time timestamp for the msg */
-
-          /* we use a separate variable for the current timestamp as strptime on
-           * solaris sometimes touches fields that are not in the format string
-           * of strptime
+          /* detect if the message is coming from last year. If its
+           * month is at least one larger than the current month. This
+           * handles both clocks that are in the future, or in the
+           * past:
+           *   in January we receive a message from December (past) => last year
+           *   in January we receive a message from February (future) => same year
+           *   in December we receive a message from January (future) => next year
            */
-          cached_localtime(&now, &nowtm);
-          tm = nowtm;
-          p = (guchar *) strptime((gchar *) date, "%b %e %H:%M:%S", &tm);
-          if (!p || (p && *p))
-            goto error;
-
-          /* In case of daylight saving let's assume that the message came under daylight saving also */
-          tm.tm_isdst = nowtm.tm_isdst;
-          tm.tm_year = nowtm.tm_year;
           if (tm.tm_mon > nowtm.tm_mon + 1)
             tm.tm_year--;
+          if (tm.tm_mon < nowtm.tm_mon - 1)
+            tm.tm_year++;
 
           /* NOTE: no timezone information in the message, assume it is local time */
           unnormalized_hour = tm.tm_hour;
-          self->timestamps[LM_TS_STAMP].time.tv_sec = cached_mktime(&tm);
-          self->timestamps[LM_TS_STAMP].time.tv_usec = usec;
+          self->timestamps[LM_TS_STAMP].tv_sec = cached_mktime(&tm);
+          self->timestamps[LM_TS_STAMP].tv_usec = usec;
         }
       else
         {
@@ -439,10 +425,10 @@ log_msg_parse_date(LogMessage *self, const guchar **data, gint *length, guchar *
     }
   if (self->timestamps[LM_TS_STAMP].zone_offset == -1)
     {
-      self->timestamps[LM_TS_STAMP].zone_offset = get_local_timezone_ofs(self->timestamps[LM_TS_STAMP].time.tv_sec);
+      self->timestamps[LM_TS_STAMP].zone_offset = get_local_timezone_ofs(self->timestamps[LM_TS_STAMP].tv_sec);
     }
-  self->timestamps[LM_TS_STAMP].time.tv_sec = self->timestamps[LM_TS_STAMP].time.tv_sec +
-                                              get_local_timezone_ofs(self->timestamps[LM_TS_STAMP].time.tv_sec) -
+  self->timestamps[LM_TS_STAMP].tv_sec = self->timestamps[LM_TS_STAMP].tv_sec +
+                                              get_local_timezone_ofs(self->timestamps[LM_TS_STAMP].tv_sec) -
                                               (tm.tm_hour - unnormalized_hour) * 3600 - self->timestamps[LM_TS_STAMP].zone_offset;
 
   *data = src;
@@ -637,7 +623,7 @@ sd_step_and_store(LogMessage *self, const guchar **data, gint *left)
  * in @self.values and dup the SD string. Parsing is affected by the bits set @flags argument.
  **/
 static gboolean
-log_msg_parse_sd(LogMessage *self, const guchar **data, gint *length, guint flags)
+log_msg_parse_sd(LogMessage *self, const guchar **data, gint *length, const MsgFormatOptions *options)
 {
   /*
    * STRUCTURED-DATA = NILVALUE / 1*SD-ELEMENT
@@ -663,7 +649,7 @@ log_msg_parse_sd(LogMessage *self, const guchar **data, gint *length, guint flag
   gchar sd_param_name[33];
 
   /* UTF-8 string */
-  gchar sd_param_value[256];
+  gchar sd_param_value[options->sdata_param_value_max + 1];
   gsize sd_param_value_len;
   gchar sd_value_name[66];
   NVHandle handle;
@@ -687,7 +673,7 @@ log_msg_parse_sd(LogMessage *self, const guchar **data, gint *length, guint flag
             goto error;
           /* read sd_id */
           pos = 0;
-          while (left && *src != ' ')
+          while (left && *src != ' ' && *src != ']')
             {
               /* the sd_id_name is max 32, the other chars are only stored in the self->sd_str*/
               if (pos < sizeof(sd_id_name) - 1)
@@ -717,7 +703,17 @@ log_msg_parse_sd(LogMessage *self, const guchar **data, gint *length, guint flag
           strcpy(sd_value_name, logmsg_sd_prefix);
           /* this strcat is safe, as sd_id_name is at most 32 chars */
           strncpy(sd_value_name + logmsg_sd_prefix_len, sd_id_name, sizeof(sd_value_name) - logmsg_sd_prefix_len);
-          sd_value_name[logmsg_sd_prefix_len + pos] = '.';
+          if (*src == ']')
+            {
+              /* Standalone sdata */
+              handle = log_msg_get_value_handle(sd_value_name);
+
+              log_msg_set_value(self, handle, "", 0);
+            }
+          else
+            {
+              sd_value_name[logmsg_sd_prefix_len + pos] = '.';
+            }
 
           /* read sd-element */
           while (left && *src != ']')
@@ -807,7 +803,6 @@ log_msg_parse_sd(LogMessage *self, const guchar **data, gint *length, guint flag
                 }
 
               handle = log_msg_get_value_handle(sd_value_name);
-              nv_registry_set_handle_flags(logmsg_registry, handle, (sd_id_len << 8) + LM_VF_SDATA);
 
               log_msg_set_value(self, handle, sd_param_value, sd_param_value_len);
             }
@@ -856,13 +851,13 @@ log_msg_parse_sd(LogMessage *self, const guchar **data, gint *length, guint flag
  * in @self. Parsing is affected by the bits set @flags argument.
  **/
 static gboolean
-log_msg_parse_legacy(MsgFormatOptions *parse_options,
+log_msg_parse_legacy(const MsgFormatOptions *parse_options,
                      const guchar *data, gint length,
                      LogMessage *self)
 {
   const guchar *src;
   gint left;
-  guchar date[32];
+  GTimeVal now;
 
   src = (const guchar *) data;
   left = length;
@@ -872,10 +867,10 @@ log_msg_parse_legacy(MsgFormatOptions *parse_options,
       return FALSE;
     }
 
+  log_msg_parse_seq(self, &src, &left);
   log_msg_parse_skip_chars(self, &src, &left, " ", -1);
-  log_msg_parse_date(self, &src, &left, date, sizeof(date), parse_options->flags & ~LP_SYSLOG_PROTOCOL, time_zone_info_get_offset(parse_options->recv_time_zone_info, time(NULL)));
-
-  if (date[0])
+  cached_g_current_time(&now);
+  if (log_msg_parse_date(self, &src, &left, parse_options->flags & ~LP_SYSLOG_PROTOCOL, time_zone_info_get_offset(parse_options->recv_time_zone_info, now.tv_sec)))
     {
       /* Expected format: hostname program[pid]: */
       /* Possibly: Message forwarded from hostname: ... */
@@ -903,8 +898,7 @@ log_msg_parse_legacy(MsgFormatOptions *parse_options,
         }
       else
         {
-
-          if ((parse_options->flags & LP_EXPECT_HOSTNAME))
+          if (!hostname_start && (parse_options->flags & LP_EXPECT_HOSTNAME))
             {
               /* Don't parse a hostname if it is local */
               /* It's a regular ol' message. */
@@ -956,7 +950,7 @@ log_msg_parse_legacy(MsgFormatOptions *parse_options,
  * Parse a message according to the latest syslog-protocol drafts.
  **/
 static gboolean
-log_msg_parse_syslog_proto(MsgFormatOptions *parse_options, const guchar *data, gint length, LogMessage *self)
+log_msg_parse_syslog_proto(const MsgFormatOptions *parse_options, const guchar *data, gint length, LogMessage *self)
 {
   /**
    *	SYSLOG-MSG      = HEADER SP STRUCTURED-DATA [SP MSG]
@@ -970,7 +964,6 @@ log_msg_parse_syslog_proto(MsgFormatOptions *parse_options, const guchar *data,
 
   const guchar *src;
   gint left;
-  guchar date[32];
   const guchar *hostname_start = NULL;
   gint hostname_len = 0;
 
@@ -988,7 +981,7 @@ log_msg_parse_syslog_proto(MsgFormatOptions *parse_options, const guchar *data,
     return FALSE;
 
   /* ISO time format */
-  if (!log_msg_parse_date(self, &src, &left, date, sizeof(date), parse_options->flags, time_zone_info_get_offset(parse_options->recv_time_zone_info, time(NULL))))
+  if (!log_msg_parse_date(self, &src, &left, parse_options->flags, time_zone_info_get_offset(parse_options->recv_time_zone_info, time(NULL))))
     return FALSE;
 
   if (!log_msg_parse_skip_space(self, &src, &left))
@@ -1023,7 +1016,7 @@ log_msg_parse_syslog_proto(MsgFormatOptions *parse_options, const guchar *data,
     return FALSE;
 
   /* structured data part */
-  if (!log_msg_parse_sd(self, &src, &left, parse_options->flags))
+  if (!log_msg_parse_sd(self, &src, &left, parse_options))
     return FALSE;
 
   /* checking if there are remaining data in log message */
@@ -1056,7 +1049,7 @@ log_msg_parse_syslog_proto(MsgFormatOptions *parse_options, const guchar *data,
 
 
 void
-syslog_format_handler(MsgFormatOptions *parse_options,
+syslog_format_handler(const MsgFormatOptions *parse_options,
                       const guchar *data, gsize length,
                       LogMessage *self)
 {
@@ -1076,6 +1069,9 @@ syslog_format_handler(MsgFormatOptions *parse_options,
   if (parse_options->flags & LP_ASSUME_UTF8)
     self->flags |= LF_UTF8;
 
+  if (parse_options->flags & LP_LOCAL)
+    self->flags |= LF_LOCAL;
+
   self->initial_parse = TRUE;
   if (parse_options->flags & LP_SYSLOG_PROTOCOL)
     success = log_msg_parse_syslog_proto(parse_options, data, length, self);
@@ -1088,6 +1084,10 @@ syslog_format_handler(MsgFormatOptions *parse_options,
       gchar buf[2048];
 
       self->timestamps[LM_TS_STAMP] = self->timestamps[LM_TS_RECVD];
+      if ((self->flags & LF_STATE_OWN_PAYLOAD) && self->payload)
+        nv_table_unref(self->payload);
+      self->flags |= LF_STATE_OWN_PAYLOAD;
+      self->payload = nv_table_new(LM_V_MAX, 16, MAX(length * 2, 256));
       log_msg_set_value(self, LM_V_HOST, "", 0);
 
       g_snprintf(buf, sizeof(buf), "Error processing log message: %.*s", (gint) length, data);
@@ -1095,7 +1095,6 @@ syslog_format_handler(MsgFormatOptions *parse_options,
       log_msg_set_value(self, LM_V_PROGRAM, "syslog-ng", 9);
       g_snprintf(buf, sizeof(buf), "%d", (int) getpid());
       log_msg_set_value(self, LM_V_PID, buf, -1);
-      log_msg_set_value(self, LM_V_MSGID, "", 0);
 
       if (self->sdata)
         {
@@ -1121,3 +1120,16 @@ syslog_format_handler(MsgFormatOptions *parse_options,
 
     }
 }
+
+void
+syslog_format_init(void)
+{
+  static gboolean handles_initialized = FALSE;
+
+  if (!handles_initialized)
+    {
+      is_synced = log_msg_get_value_handle(".SDATA.timeQuality.isSynced");
+      cisco_seqid = log_msg_get_value_handle(".SDATA.meta.sequenceId");
+      handles_initialized = TRUE;
+    }
+}
